#include <Astri_DAQ.h>
#include <Astri_Packet.h>

#define OUT_ROOT std::string(std::getenv("BIAS_PREFIX")).append("/ArchiveRAW/")

AstriDAQ::AstriDAQ(BaseProtocol &socket, BaseArchiver &archiver, BaseProvider &provider, BaseChecker &checker, BasePacket &packet, BaseMonitor &monitor){
	this->socket = &socket;
	this->archiver = &archiver;
	this->provider = &provider;
	this->checker = &checker;
	this->packet = reinterpret_cast <AstriPacket *>(&packet);
	this->runID = 0;
	this->currentState = STOP;
	this->monitor = &monitor;
}

std::string AstriDAQ::getStateStr(){
	switch(this->currentState){
	case STOP:
		return std::string("STOP");
	case IDLE:
		return std::string("IDLE");
	case RUN:
		return std::string("RUN");
	case READY:
		return std::string("READY");
	default:
		return std::string("NONE");
	}
}

void AstriDAQ::setRunID(int runId){
	this->runID = runId;
}
int AstriDAQ::getRunID(){
	return this->runID;
}
void AstriDAQ::incrementRunID(){
	this->runID +=1;
}

void AstriDAQ::stop(){
}

void AstriDAQ::init(){
	this->getSocketPtr()->connectToClient();
	this->setCurrentState(IDLE);
	archiver->open();
	provider->open();
}



void AstriDAQ::switchState(int newState){
	std::cout << "Switching from " << this->currentState << " to -> " << newState << std::endl;
	std::string dest;
	switch(this->currentState){
	case IDLE:{
		switch(newState){
		case IDLE:
			archiver->close();
			this->incrementRunID();
			this->setCurrentState(newState);
			dest = std::string(OUT_ROOT).append(std::to_string(runID)).append("_").append(getStateStr().append(".raw"));
			archiver->setDest(dest);
			break;
		case RUN:
			archiver->close();
			this->setCurrentState(newState);
			dest = std::string(OUT_ROOT).append(std::to_string(runID)).append("_").append(getStateStr().append(".raw"));
			archiver->setDest(dest);
			break;
		case STOP:
			this->setCurrentState(newState);
			provider->close();
			archiver->close();
			break;
		}
		break;
	}
	case RUN:{
		switch(newState){
		case IDLE:
			archiver->close();
			this->incrementRunID();
			this->setCurrentState(newState);
			dest = std::string(OUT_ROOT).append(std::to_string(runID)).append("_").append(getStateStr().append(".raw"));
			archiver->setDest(dest);
			break;
		case RUN:
			archiver->close();
			this->incrementRunID();
			dest = std::string(OUT_ROOT).append(std::to_string(runID)).append("_").append(getStateStr().append(".raw"));
			archiver->setDest(dest);
			break;
		case STOP:
			this->setCurrentState(newState);
			std::cout << "Closing provider and archiver..." << std::endl;
			archiver->close();
			provider->close();
			break;
		}
	}
	}
}

void AstriDAQ::start(int initialState){
	switchState(initialState);
	while(currentState!= STOP){
		int rec = socket->rcvPacketFromCli(*packet);
		if(rec > 0){
			this->deliverPacket();
			if(changeStateFlag){
				this->switchState(getNextState());
				this->setChangeStateFlag(false);
			}
		}
		else if(rec < 0){
			this->switchState(STOP);
			monitor->printStats();
		}
		else{
			this->switchState(STOP); //TBD
			monitor->printStats();
		}
	}
}

int AstriDAQ::deliverPacket(){
	//-------- CHECK CORRECTNESS
	AstriPacket &pack = *reinterpret_cast <AstriPacket *>(packet);
	if(checker->checkPacket(packet)){
		if(pack.getType()== 15){ //NOTIF PACKET
			unsigned short cmd =((pack.getBufferPtr()[8]) << 8) + (pack.getBufferPtr()[9]);
			int newState = (cmd == 43690)*RUN + (cmd == 21845)*IDLE;
			this->setNextState(newState);
			this->setChangeStateFlag(true);
		}
	}
	else{
		std::cout << "----- PACKET ERROR ------" << std::endl;
		std::cout << packet->toString() << std::endl;
		std::cout << "------------------------------" << std::endl;
	}
	archiver->write(*packet); // write anyway
	provider->write(*packet); // write anyway
	monitor->monit(*packet);
	return 0;
}

