//============================================================================
// Name        : Receiver.cpp
// Author      : Valerio Pastore

//============================================================================


#include <Astri_DAQ.h>
#include <DAQ_Configurator.h>

#include <signal.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>

AstriDAQ *daq; // I'm sorry

void my_handler(int s){
	daq->stop(); // TBD.. now empty
	daq->getMonitorPtr()->printStats();
	std::cout << "Termination..." << std::endl;
	daq->switchState(AstriDAQ::STOP);
    std::cout << "Terminated by CTRL-C\n" << std::endl;
    exit(1);
}


using namespace std;


int main(int argc, char **argv){
// ----- ELAB INPUT PARAMS
	DAQConfigurator *daqConfiguration = new DAQConfigurator(argc, argv);
	DAQConfigurator &daqConfig = *daqConfiguration;

	BaseArchiver * archiver = daqConfig.createArchiver(argc, argv);
	BaseProvider * provider = daqConfig.createProvider(argc, argv);
	BasePacket * packet = daqConfig.createPacket(argc, argv);
	BaseChecker * checker = daqConfig.createChecker(argc, argv);
	BaseMonitor * monitor = daqConfig.createMonitor(argc, argv);
	BaseProtocol * socket = daqConfig.createProtocol(argc, argv);

	cout << "Starting program with params: " << endl;
	cout << daqConfig.toString() << endl;
	cout << endl << "-------------------------------" << endl;

	daq = new AstriDAQ(*socket, *archiver, *provider, *checker, *packet, *monitor);

	struct sigaction sigIntHandler;
	sigIntHandler.sa_handler = my_handler;
	sigemptyset(&sigIntHandler.sa_mask);
	sigIntHandler.sa_flags = 0;
	sigaction(SIGINT, &sigIntHandler, NULL);

	daq->init();
	daq->start(BaseDAQ::IDLE);

    std::cout << "Terminated normally\n" << std::endl;
	return 0;
}
