#include <Sender_Configurator.h>
#include <tclap/CmdLine.h>

#include <TCP_Protocol_Configurator.h>
#include <UDP_Protocol_Configurator.h>
#include <Astri_Monitor_Configurator.h>
#include <Astri_Packet.h>


#define PACKETS_PATH std::string(std::getenv("BIAS_PREFIX")).append("/config/packets/xml_packets/")
#define RAW_PACKETS_PATH std::string(std::getenv("BIAS_PREFIX")).append("/config/packets/raw test packets")


SenderConfigurator::SenderConfigurator(){
	setProtocol("udp");
	setPacket("astri_tm");
	setMonitor("astri");
	setMode("playback");
	setSource(RAW_PACKETS_PATH.append("/config/packets/raw test packets/test.raw"));
}


SenderConfigurator::SenderConfigurator(int argc, char ** argv){
	try {
		TCLAP::CmdLine cmd("Usage Description WIP", ' ', "1");
		cmd.ignoreUnmatched(true);
		TCLAP::ValueArg<std::string> protocol("","protocol","communication protocol",false,"udp","[tcp|udp|.. |supported protocols]");
		cmd.add( protocol );
		TCLAP::ValueArg<std::string> packet("","packet","the file name of the packet structure",false,"astri_tm","[astri_tm|..]");
		cmd.add( packet );
		TCLAP::ValueArg<std::string> monitor("","monitor","the packet monitor",false,"astri","[astri|..]");
		cmd.add( monitor );
		TCLAP::ValueArg<std::string> mode("","mode","Sender modality",false,"playback","[playback|realtime..]");
		cmd.add( mode );
		TCLAP::ValueArg<std::string> source("","source","The packets path source",false,"../config/packets/raw test packets/test.raw","The file path");
		cmd.add( source );
		cmd.parse( argc, argv );
		setProtocol(protocol.getValue());
		setPacket(packet.getValue());
		setMonitor(monitor.getValue());
		setMode(mode.getValue());
		setSource(source.getValue());

	}catch (TCLAP::ArgException &e){  // catch any exceptions
		std::cerr << "error: " << e.error() << " for arg " << e.argId() << std::endl;
		exit(EXIT_FAILURE);
	}
}


BasePacket * SenderConfigurator::createPacket(int argc, char ** argv){
	std::string packet_path(PACKETS_PATH);
	packet_path.append(getPacket().append(".xml"));
	return new AstriPacket(packet_path);
}

BaseMonitor * SenderConfigurator::createMonitor(int argc, char ** argv){
	if(getMonitor().compare("astri") == 0){
		AstriMonitorConfigurator conf(argc,argv);
		return conf.createMonitor();
	}
	throw std::invalid_argument("Invalid Monitor");
}

BaseProtocol * SenderConfigurator::createProtocol(int argc, char ** argv){
	if(getProtocol().compare("tcp") == 0){
		TCPProtocolConfigurator conf(argc,argv);
		return conf.createProtocol();
	}
	if(getProtocol().compare("udp") == 0){
		UDPProtocolConfigurator conf(argc,argv);
		return conf.createProtocol();
	}
	throw std::invalid_argument("Invalid protocol");
}
